#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import subprocess
import psutil
import json
from ikabot.config import *
from ikabot.helpers.signals import deactivate_sigint

def set_child_mode(session):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	"""
	session.padre = False
	deactivate_sigint()
	session.login()

def run(command):
	return subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout.read()

def updateProcessList(session, programprocesslist = []):
	"""This function will return data about all the active ikabot processes. If it is passed the ``programprocesslist`` argument, it will write new processes from that list to the .ikabot file
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	programprocesslist : list[dict]
		a list of dictionaries containing relevant data about a running ikabot process ('pid', 'proxies' and 'action')

	Returns
	-------
	runningIkabotProcessList : list[dict]
		a list of dictionaries containing relevant data about a running ikabot process ('pid', 'proxies' and 'action')
	"""
	# read from file
	sessionData = session.getSessionData()
	try:
		fileList = sessionData['processList']
	except KeyError:
		fileList = []

	# check it's still running
	runningIkabotProcessList = []
	ika_process = 'python.exe' if isWindows else 'ikabot'
	for process in fileList:
		try:
			proc = psutil.Process(pid = process['pid'])
		except psutil.NoSuchProcess:
			continue

		# windows doesn't support the status method
		isAlive = True if isWindows else proc.status() != 'zombie'

		if proc.name() == ika_process and isAlive:
			runningIkabotProcessList.append(process)

	# add new to the list and write to file only if it's given
	for process in programprocesslist:
		if process not in runningIkabotProcessList:
			runningIkabotProcessList.append(process)

	# write to file
	sessionData['processList'] = runningIkabotProcessList
	session.setSessionData(sessionData)

	return runningIkabotProcessList



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.planRoutes import executeRoutes
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCity
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.varios import addDot
from ikabot.helpers.resources import *

t = gettext.translation('sendResources',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def sendResources(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		routes = []
		while True:

			banner()
			print(_('Origin city:'))
			try:
				cityO = chooseCity(session)
			except KeyboardInterrupt:
				if routes:
					print(_('Send shipment? [Y/n]'))
					rta = read(values=['y', 'Y', 'n', 'N', ''])
					if rta.lower() != 'n':
						break
				event.set()
				return

			banner()
			print(_('Destination city'))
			cityD = chooseCity(session, foreign=True)
			idIsland = cityD['islandId']

			if cityO['id'] == cityD['id']:
				continue

			resources_left = cityO['recursos']
			for route in routes:
				(origin_city, destination_city, __, *toSend) = route
				if origin_city['id'] == cityO['id']:
					for i in range(len(materials_names)):
						resources_left[i] -= toSend[i]

				# the destination city might be from another player
				if cityD['propia'] and destination_city['id'] == cityD['id']:
					for i in range(len(materials_names)):
						cityD['freeSpaceForResources'][i] -= toSend[i]

			banner()
			# the destination city might be from another player
			if cityD['propia']:
				msg = ''
				for i in range(len(materials_names)):
					if resources_left[i] > cityD['freeSpaceForResources'][i]:
						msg += '{} more {}\n'.format(addDot(cityD['freeSpaceForResources'][i]), materials_names[i].lower())

				if len(msg) > 0:
					print(_('You can store just:\n{}').format(msg))

			print(_('Available:'))
			for i in range(len(materials_names)):
				print('{}:{} '.format(materials_names[i], addDot(resources_left[i])), end='')
			print('')

			print(_('Send:'))
			try:
				max_name = max( [ len(material) for material in materials_names ] )
				send = []
				for i in range(len(materials_names)):
					material_name = materials_names[i]
					pad = ' ' * ( max_name - len(material_name) )
					val = askForValue(_('{}{}:'.format(pad, material_name)), resources_left[i])
					send.append(val)
			except KeyboardInterrupt:
				continue
			if sum(send) == 0:
				continue

			banner()
			print(_('About to send from {} to {}').format(cityO['cityName'], cityD['cityName']))
			for i in range(len(materials_names)):
				if send[i] > 0:
					print('{}:{} '.format(materials_names[i], addDot(send[i])), end='')
			print('')

			print(_('Proceed? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() != 'n':
				route = (cityO, cityD, idIsland, *send)
				routes.append(route)
				print(_('Create another shipment? [y/N]'))
				rta = read(values=['y', 'Y', 'n', 'N', ''])
				if rta.lower() != 'y':
					break
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nSend resources\n')

	setInfoSignal(session, info)
	try:
		executeRoutes(session, routes)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.varios import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.varios import addDot
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.resources import getAvailableResources

t = gettext.translation('trainArmy',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def getBuildingInfo(session, city, trainTroops):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	trainTroops : bool

	Returns
	-------
	response : dict
	"""
	view = 'barracks' if trainTroops else 'shipyard'
	params = {'view': view, 'cityId': city['id'], 'position': city['pos'], 'backgroundView': 'city', 'currentCityId': city['id'], 'actionRequest': actionRequest, 'ajax': '1'}
	data = session.post(params=params)
	return json.loads(data, strict=False)

def train(session, city, trainings, trainTroops):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	trainings : list[dict]
	trainTroops : bool
	"""
	templateView = 'barracks' if trainTroops else 'shipyard'
	function = 'buildUnits' if trainTroops else 'buildShips'
	payload = {'action': 'CityScreen', 'function': function, 'actionRequest': 'REQUESTID', 'cityId': city['id'], 'position': city['pos'], 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': templateView, 'ajax': '1'}
	for training in trainings:
		payload[ training['unit_type_id'] ] = training['train']
	session.post(payloadPost=payload)

def waitForTraining(session, city, trainTroops):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	trainTroops : bool
	"""
	data = getBuildingInfo(session, city, trainTroops)
	html = data[1][1][1]
	seconds = re.search(r'\'buildProgress\', (\d+),', html)
	if seconds:
		seconds = seconds.group(1)
		seconds = int(seconds) - data[0][1]['time']
		wait(seconds + 5)

def planTrainings(session, city, trainings, trainTroops):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	trainings : list[list[dict]]
	trainTroops : bool
	"""
	buildingPos = city['pos']

	# trainings might be divided in multriple rounds
	while True:

		# total number of units to create
		total = sum( [ unit['cantidad'] for training in trainings for unit in training ] )
		if total == 0:
			return

		for training in trainings:
			waitForTraining(session, city, trainTroops)
			html = session.get(city_url + city['id'])
			city = getCity(html)
			city['pos'] = buildingPos

			resourcesAvailable = city['recursos'].copy()
			resourcesAvailable.append( city['ciudadanosDisp'] )

			# for each unit type in training
			for unit in training:

				# calculate how many units can actually be trained based on the resources available
				unit['train'] = unit['cantidad']

				for i in range(len(materials_names_english)):
					material_name = materials_names_english[i].lower()
					if material_name in unit['costs']:
						limiting = resourcesAvailable[i] // unit['costs'][material_name]
						unit['train'] = min(unit['train'], limiting)

				if 'citizens' in unit['costs']:
					limiting = resourcesAvailable[len(materials_names_english)] // unit['costs']['citizens']
					unit['train'] = min(unit['train'], limiting)

				# calculate the resources that will be left
				for i in range(len(materials_names_english)):
					material_name = materials_names_english[i].lower()
					if material_name in unit['costs']:
						resourcesAvailable[i] -= unit['costs'][material_name] * unit['train']

				if 'citizens' in unit['costs']:
					resourcesAvailable[len(materials_names_english)] -= unit['costs']['citizens'] * unit['train']

				unit['cantidad'] -= unit['train']

			# amount of units that will be trained
			total = sum( [ unit['train'] for unit in training ] )
			if total == 0:
				msg = _('It was not possible to finish the training due to lack of resources.')
				sendToBot(session, msg)
				return

			train(session, city, training, trainTroops)

def generateArmyData(units_info):
	"""
	Parameters
	----------
	units_info : dict

	Returns
	-------
	units : list[dict]
	"""
	i = 1
	units = []
	while 'js_barracksSlider{:d}'.format(i) in units_info:
		# {"identifier":"phalanx","unit_type_id":303,"costs":{"citizens":1,"wood":27,"sulfur":30,"upkeep":3,"completiontime":71.169695412658},"local_name":"Hoplita"}
		info = units_info['js_barracksSlider{:d}'.format(i)]['slider']['control_data']
		info = json.loads(info, strict=False)
		units.append(info)
		i += 1
	return units

def trainArmy(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()

		print(_('Do you want to train troops (1) or ships (2)?'))
		rta = read(min=1, max=2)
		trainTroops = rta == 1
		banner()

		if trainTroops:
			print(_('In what city do you want to train the troops?'))
		else:
			print(_('In what city do you want to train the fleet?'))
		city = chooseCity(session)
		banner()

		lookfor = 'barracks' if trainTroops else 'shipyard'
		for i in range(len(city['position'])):
			if city['position'][i]['building'] == lookfor:
				city['pos'] = str(i)
				break
		else:
			if trainTroops:
				print(_('Barracks not built.'))
			else:
				print(_('Shipyard not built.'))
			enter()
			event.set()
			return

		data = getBuildingInfo(session, city, trainTroops)

		units_info = data[2][1]
		units = generateArmyData(units_info)

		maxSize = max( [ len(unit['local_name']) for unit in units ] )

		tranings = []
		while True:
			units = generateArmyData(units_info)
			print(_('Train:'))
			for unit in units:
				pad = ' ' * ( maxSize - len(unit['local_name']) )
				amount = read(msg='{}{}:'.format(pad, unit['local_name']), min=0, empty=True)
				if amount == '':
					amount = 0
				unit['cantidad'] = amount

			# calculate costs
			cost = [0] * ( len(materials_names_english) + 3 )
			for unit in units:
				for i in range(len(materials_names_english)):
					material_name = materials_names_english[i].lower()
					if material_name in unit['costs']:
						cost[i] += unit['costs'][material_name] * unit['cantidad']

				if 'citizens' in unit['costs']:
					cost[len(materials_names_english)+0] += unit['costs']['citizens'] * unit['cantidad']
				if 'upkeep' in unit['costs']:
					cost[len(materials_names_english)+1] += unit['costs']['upkeep'] * unit['cantidad']
				if 'completiontime' in unit['costs']:
					cost[len(materials_names_english)+2] += unit['costs']['completiontime'] * unit['cantidad']

			print(_('\nTotal cost:'))
			for i in range(len(materials_names_english)):
				if cost[i] > 0:
					print('{}: {}'.format(materials_names_english[i], addDot(cost[i])))
			if cost[len(materials_names_english)+0] > 0:
				print(_('Citizens: {}').format(addDot(cost[len(materials_names_english)+0])))
			if cost[len(materials_names_english)+1] > 0:
				print(_('Maintenance: {}').format(addDot(cost[len(materials_names_english)+1])))
			if cost[len(materials_names_english)+2] > 0:
				print(_('Duration: {}').format(daysHoursMinutes(int(cost[len(materials_names_english)+2]))))

			print(_('\nProceed? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				event.set()
				return

			tranings.append(units)

			if trainTroops:
				print(_('\nDo you want to train more troops when you finish? [y/N]'))
			else:
				print(_('\nDo you want to train more fleets when you finish? [y/N]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'y':
				banner()
				continue
			else:
				break

		# calculate if the city has enough resources
		resourcesAvailable = city['recursos'].copy()
		resourcesAvailable.append( city['ciudadanosDisp'] )

		for training in tranings:
			for unit in training:

				for i in range(len(materials_names_english)):
					material_name = materials_names_english[i].lower()
					if material_name in unit['costs']:
						resourcesAvailable[i] -= unit['costs'][material_name] * unit['cantidad']

				if 'citizens' in unit['costs']:
					resourcesAvailable[len(materials_names_english)] -= unit['costs']['citizens'] * unit['cantidad']

		not_enough = [ elem for elem in resourcesAvailable if elem < 0 ] != []

		if not_enough:
			print(_('\nThere are not enough resources:'))
			for i in range(len(materials_names_english)):
				if resourcesAvailable[i] < 0:
					print('{}:{}'.format(materials_names[i], addDot(resourcesAvailable[i]*-1)))

			if resourcesAvailable[len(materials_names_english)] < 0:
				print(_('Citizens:{}').format(addDot(resourcesAvailable[len(materials_names_english)]*-1)))

			print(_('\nProceed anyway? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				event.set()
				return

		if trainTroops:
			print(_('\nThe selected troops will be trained.'))
		else:
			print(_('\nThe selected fleet will be trained.'))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	if trainTroops:
		info = _('\nI train troops in {}\n').format(city['cityName'])
	else:
		info = _('\nI train fleets in {}\n').format(city['cityName'])
	setInfoSignal(session, info)
	try:
		planTrainings(session, city, tranings, trainTroops)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import traceback
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import getIdsOfCities
from ikabot.helpers.varios import daysHoursMinutes
from ikabot.helpers.getJson import getCity
from ikabot.helpers.resources import *
from ikabot.helpers.botComm import *

t = gettext.translation('alertLowWine',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def alertLowWine(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		if checkTelegramData(session) is False:
			event.set()
			return
		banner()
		hours = read(msg=_('How many hours should be left until the wine runs out in a city so that it\'s alerted?'), min=1)
		print(_('It will be alerted when the wine runs out in less than {:d} hours in any city').format(hours))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI alert if the wine runs out in less than {:d} hours\n').format(hours)
	setInfoSignal(session, info)
	try:
		do_it(session, hours)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def do_it(session, hours):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	hours : int
	"""
	ids, cities = getIdsOfCities(session)
	while True:
		# getIdsOfCities is called on a loop because the amount of cities may change
		ids, cities_new = getIdsOfCities(session)
		if len(cities_new) != len(cities):
			cities = cities_new

		for cityId in cities:
			if 'reported' not in cities[cityId]:
				cities[cityId]['reported'] = False

		for cityId in cities:
			html = session.get(city_url + cityId)
			city = getCity(html)

			# if the city doesn't even have a tavern built, ignore it
			if 'tavern' not in [ building['building'] for building in city['position'] ]:
				continue

			consumption_per_hour = city['consumo']

			# is a wine city
			if cities[cityId]['tradegood'] == '1':
				wine_production = getProductionPerSecond(session, cityId)[1]
				wine_production = wine_production * 60 * 60
				if consumption_per_hour > wine_production:
					consumption_per_hour -= wine_production
				else:
					continue

			consumption_per_seg = Decimal(consumption_per_hour) / Decimal(3600)
			wine_available = city['recursos'][1]

			if consumption_per_seg == 0:
				if cities[cityId]['reported'] is False:
					msg = _('The city {} is not consuming wine!').format(city['name'])
					sendToBot(session, msg)
					cities[cityId]['reported'] = True
				continue

			seconds_left = Decimal(wine_available) / Decimal(consumption_per_seg)
			if seconds_left < hours*60*60:
				if cities[cityId]['reported'] is False:
					time_left = daysHoursMinutes(seconds_left)
					msg = _('In {}, the wine will run out in {}').format(time_left, city['name'])
					sendToBot(session, msg)
					cities[cityId]['reported'] = True
			else:
				cities[cityId]['reported'] = False

		time.sleep(20*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import json
import math
import random
import gettext
import traceback
import threading
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.web.session import normal_get
from ikabot.helpers.planRoutes import *
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.resources import getAvailableResources
t = gettext.translation('constructionList',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

sendResources = True
expand = True

def getConstructionTime(session, html, building_position_id):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	html : string
	building_position_id : int

	Returns
	-------
	seconds_to_wait : int
	"""
	city = getCity(html)
	building = city['position'][building_position_id]
	final_time = re.search(r'"endUpgradeTime":(\d{10})', html)
	if final_time is None:
		msg = _('{}: I don\'t wait anything so that {} gets to the level {:d}').format(city['cityName'], building['name'], building['level'])
		sendToBotDebug(session, msg, debugON_constructionList)
		return 0

	current_time    = int( time.time() )
	final_time      = int( final_time.group(1) )
	seconds_to_wait = final_time - current_time
	if seconds_to_wait <= 0:
		seconds_to_wait = 0

	msg = _('{}: I wait {:d} seconds so that {} gets to the level {:d}').format(city['cityName'], seconds_to_wait, building['name'], building['level'] + 1)
	sendToBotDebug(session, msg, debugON_constructionList)

	return seconds_to_wait

def waitForConstruction(session, city_id, building_position):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city_id : int
	building_position : int

	Returns
	-------
	city : dict
	"""
	seconds_to_finish_building = 1
	while seconds_to_finish_building > 0:
		html = session.get(city_url + city_id)
		seconds_to_finish_building = getConstructionTime(session, html, building_position)
		wait(seconds_to_finish_building + 5)
	html = session.get(city_url + city_id)
	city = getCity(html)
	building = city['position'][building_position]
	msg = _('{}: The building {} reached the level {:d}.').format(city['cityName'], building['name'], building['level'])
	sendToBotDebug(session, msg, debugON_constructionList)
	return city

def expandBuilding(session, cityId, building, waitForResources):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	cityId : int
	building : dict
	waitForResources : bool
	"""
	current_level = building['level']
	if building['isBusy']:
		current_level += 1
	levels_to_upgrade = building['upgradeTo'] - current_level
	position = building['position']

	time.sleep(random.randint(5,15)) # to avoid race conditions with sendResourcesNeeded

	for lv in range(levels_to_upgrade):
		city = waitForConstruction(session, cityId, position)
		building = city['position'][position]

		if building['canUpgrade'] is False and waitForResources is True:
			while building['canUpgrade'] is False:
				time.sleep(60)
				seconds = getMinimumWaitingTime(session)
				html = session.get(city_url + cityId)
				city = getCity(html)
				building = city['position'][position]
				# if no ships are comming, exit no matter if the building can or can't upgrade
				if seconds == 0:
					break
				wait(seconds + 5)

		if building['canUpgrade'] is False:
			msg  = _('City:{}\n').format(city['cityName'])
			msg += _('Building:{}\n').format(building['name'])
			msg += _('The building could not be completed due to lack of resources.\n')
			msg += _('Missed {:d} levels').format(levels_to_upgrade - lv)
			sendToBot(session, msg)
			return

		url = 'action=CityScreen&function=upgradeBuilding&actionRequest={}&cityId={}&position={:d}&level={}&activeTab=tabSendTransporter&backgroundView=city&currentCityId={}&templateView={}&ajax=1'.format(actionRequest, cityId, position, building['level'], cityId, building['building'])
		resp = session.post(url)
		html = session.get(city_url + cityId)
		city = getCity(html)
		building = city['position'][position]
		if building['isBusy'] is False:
			msg  = _('{}: The building {} was not extended').format(city['cityName'], building['name'])
			sendToBot(session, msg)
			sendToBot(session, resp)
			return

		msg = _('{}: The building {} is being extended to level {:d}.').format(city['cityName'], building['name'], building['level']+1)
		sendToBotDebug(session, msg, debugON_constructionList)

	msg = _('{}: The building {} finished extending to level: {:d}.').format(city['cityName'], building['name'], building['level']+1)
	sendToBotDebug(session, msg, debugON_constructionList)

def getCostsReducers(city):
	"""
	Parameters
	----------
	city : dict

	Returns
	-------
	reducers_per_material_level : dict[int, int]
	"""
	reducers_per_material = [0] * len(materials_names)
	assert len(reducers_per_material) == 5

	for building in city['position']:
		if building['name'] == 'empty':
			continue
		lv = building['level']
		if building['building'] == 'carpentering':
			reducers_per_material[0] = lv
		elif building['building'] == 'vineyard':
			reducers_per_material[1] = lv
		elif building['building'] == 'architect':
			reducers_per_material[2] = lv
		elif building['building'] == 'optician':
			reducers_per_material[3] = lv
		elif building['building'] == 'fireworker':
			reducers_per_material[4] = lv
	return reducers_per_material

def getResourcesNeeded(session, city, building, current_level, final_level):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	building : dict
	current_level : int
	final_level : int

	Returns
	-------
	costs_per_material : dict[int, int]
	"""
	# get html with information about buildings
	building_detail_url = 'view=buildingDetail&buildingId=0&helpId=1&backgroundView=city&currentCityId={}&templateView=ikipedia&actionRequest={}&ajax=1'.format(city['id'], actionRequest)
	building_detail_response = session.post(building_detail_url)
	building_detail = json.loads(building_detail_response, strict=False)
	building_html = building_detail[1][1][1]

	# get html with information about buildings costs
	regex_building_detail = r'<div class="(?:selected)? button_building '+ re.escape(building['building']) + r'"\s*onmouseover="\$\(this\)\.addClass\(\'hover\'\);" onmouseout="\$\(this\)\.removeClass\(\'hover\'\);"\s*onclick="ajaxHandlerCall\(\'\?(.*?)\'\);'
	match = re.search(regex_building_detail, building_html)
	building_costs_url = match.group(1)
	building_costs_url += 'backgroundView=city&currentCityId={}&templateView=buildingDetail&actionRequest={}&ajax=1'.format(city['id'], actionRequest)
	building_costs_response = session.post(building_costs_url)
	building_costs = json.loads(building_costs_response, strict=False)
	html_costs = building_costs[1][1][1]

	# if the user has all the resource saving studies, we save that in the session data (one less request)
	sessionData = session.getSessionData()
	if 'reduccion_inv_max' in sessionData:
		costs_reduction = 14
	else:
		# get the studies
		url = 'view=noViewChange&researchType=economy&backgroundView=city&currentCityId={}&templateView=researchAdvisor&actionRequest={}&ajax=1'.format(city['id'], actionRequest)
		rta = session.post(url)
		rta = json.loads(rta, strict=False)
		studies = rta[2][1]['new_js_params']
		studies = json.loads(studies, strict=False)
		studies = studies['currResearchType']

		# look for resource saving studies
		costs_reduction = 0
		for study in studies:
			if studies[study]['liClass'] != 'explored':
				continue
			link = studies[study]['aHref']
			if '2020' in link:
				costs_reduction += 2
			elif '2060' in link:
				costs_reduction += 4
			elif '2100' in link:
				costs_reduction += 8

		# if the user has all the resource saving studies, save that in the session data
		if costs_reduction == 14:
			sessionData['reduccion_inv_max'] = True
			session.setSessionData(sessionData)

	# calculate cost reductions
	costs_reduction /= 100
	costs_reduction = 1 - costs_reduction

	# get buildings that reduce the cost of upgrades
	costs_reductions = getCostsReducers(city)

	# get the type of resources that this upgrade will cost (wood, marble, etc)
	resources_types = re.findall(r'<th class="costs"><img src="skin/resources/icon_(.*?)\.png"/></th>', html_costs)[:-1]

	# get the actual cost of each upgrade
	matches = re.findall(r'<td class="level">\d+</td>(?:\s+<td class="costs">.*?</td>)+', html_costs)

	# calculate the cost of the entire upgrade, taking into account all the possible reductions
	final_costs = [0] * len(materials_names)
	levels_to_upgrade = 0
	for match in matches:
		lv = re.search(r'"level">(\d+)</td>', match).group(1)
		lv = int(lv)

		if lv <= current_level:
			continue
		if lv > final_level:
			break

		levels_to_upgrade += 1

		# get the costs for the current level
		costs = re.findall(r'<td class="costs">([\d,\.]*)</td>', match)
		for i in range(len(costs)):
			resource_type = resources_types[i]
			for j in range(len(materials_names_tec)):
				name = materials_names_tec[j]
				if resource_type == name:
					resource_index = j
					break

			# get the cost of the current resource type
			cost = costs[i]
			cost = cost.replace(',', '').replace('.', '')
			cost = 0 if cost == '' else int(cost)

			# calculate all the reductions
			real_cost = Decimal(cost)
			# investigation reduction
			original_cost = Decimal(real_cost) / Decimal(costs_reduction)
			# special building reduction
			real_cost -= Decimal(original_cost) * (Decimal(costs_reductions[resource_index]) / Decimal(100))

			final_costs[resource_index] += math.ceil(real_cost)

	if levels_to_upgrade < final_level - current_level:
		print(_('This building only allows you to expand {:d} more levels').format(levels_to_upgrade))
		msg = _('Expand {:d} levels? [Y/n]:').format(levels_to_upgrade)
		rta = read(msg=msg, values=['Y', 'y', 'N', 'n', ''])
		if rta.lower() == 'n':
			return [-1,-1,-1,-1,-1]

	return final_costs

def sendResourcesNeeded(session, destination_city_id, city_origins, missing_resources):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	destination_city_id : int
	city_origins : dict
	missing_resources : dict[int, int]
	"""
	#set_child_mode(s)

	info = _('\nTransport resources to upload building\n')

	try:
		routes = []
		html = session.get(city_url + destination_city_id)
		cityD = getCity(html)
		for i in range(len(materials_names)):
			missing = missing_resources[i]
			if missing <= 0:
				continue

			# send the resources from each origin city
			for cityOrigin in city_origins[i]:
				if missing == 0:
					break

				available = cityOrigin['recursos'][i]
				send = min(available, missing)
				missing -= send
				toSend = [0] * len(materials_names)
				toSend[i] = send
				route = (cityOrigin, cityD, cityD['islandId'], *toSend)
				routes.append(route)
		executeRoutes(session, routes)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
		# no s.logout() because this is a thread, not a process

def chooseResourceProviders(session, cities_ids, cities, city_id, resource, missing):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	cities_ids : list[int]
	cities : dict[int, dict]
	city_id : int
	resource : int
	missing : int
	"""
	global sendResources
	sendResources = True
	global expand
	expand = True

	banner()
	print(_('From what cities obtain {}?').format(materials_names[resource].lower()))

	tradegood_initials = [ material_name[0] for material_name in materials_names ]
	maxName = max ([len(cities[city]['name']) for city in cities if cities[city]['id'] != city_id])

	origin_cities = []
	total_available = 0
	for cityId in cities_ids:
		if cityId == city_id:
			continue

		html = session.get(city_url + cityId)
		city = getCity(html)

		available = city['recursos'][resource]
		if available == 0:
			continue

		# ask the user it this city should provide resources
		tradegood_initial = tradegood_initials[ int( cities[cityId]['tradegood'] ) ]
		pad = ' ' * (maxName - len(cities[cityId]['name']))
		msg = '{}{} ({}): {} [Y/n]:'.format(pad, cities[cityId]['name'], tradegood_initial, addDot(available))
		choice = read(msg=msg, values=['Y', 'y', 'N', 'n', ''])
		if choice.lower() == 'n':
			continue

		# if so, save the city and calculate the total amount resources to send
		total_available += available
		origin_cities.append(city)
		# if we have enough resources, return
		if total_available >= missing:
			return origin_cities

	# if we reach this part, there are not enough resources to expand the building
	print(_('\nThere are not enough resources.'))

	if len(origin_cities) > 0:
		print(_('\nSend the resources anyway? [Y/n]'))
		choice = read(values=['y', 'Y', 'n', 'N', ''])
		if choice.lower() == 'n':
			sendResources = False

	print(_('\nTry to expand the building anyway? [y/N]'))
	choice = read(values=['y', 'Y', 'n', 'N', ''])
	if choice.lower() == 'n' or choice == '':
		expand = False

	return origin_cities

def sendResourcesMenu(session, city_id, missing):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city_id : int
	missing : list[int, int]
	"""
	cities_ids, cities = getIdsOfCities(session)
	origins = {}
	# for each missing resource, choose providers
	for resource in range(len(missing)):
		if missing[resource] <= 0:
			continue

		origin_cities = chooseResourceProviders(session, cities_ids, cities, city_id, resource, missing[resource])
		if sendResources is False and expand:
			print(_('\nThe building will be expanded if possible.'))
			enter()
			return
		elif sendResources is False:
			return
		origins[resource] = origin_cities

	if expand:
		print(_('\nThe resources will be sent and the building will be expanded if possible.'))
	else:
		print(_('\nThe resources will be sent.'))

	enter()

	# create a new thread to send the resources
	thread = threading.Thread(target=sendResourcesNeeded, args=(session, city_id, origins, missing,))
	thread.start()

def getBuildingToExpand(session, cityId):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	cityId : int

	Returns
	-------
	building : dict
	"""
	html = session.get(city_url + cityId)
	city = getCity(html)

	banner()
	# show the buildings available to expand (ignore empty spaces)
	print(_('Which building do you want to expand?\n'))
	print(_('(0)\t\texit'))
	buildings = [ building for building in city['position'] if building['name'] != 'empty' ]
	for i in range(len(buildings)):
		building = buildings[i]

		level = building['level']
		if level < 10:
			level = ' ' + str(level)
		else:
			level = str(level)
		if building['isBusy']:
			level = level + '+'
		print(_('({:d})\tlv:{}\t{}').format(i+1, level, building['name']))

	selected_building_id = read(min=0, max=len(buildings))
	if selected_building_id == 0:
		return None

	building = buildings[selected_building_id - 1]

	current_level = int(building['level'])
	# if the building is being expanded, add 1 level
	if building['isBusy']:
		current_level += 1

	banner()
	print(_('building:{}').format(building['name']))
	print(_('current level:{}').format(current_level))

	final_level = read(min=current_level, msg=_('increase to level:'))
	building['upgradeTo'] = final_level

	return building

def constructionList(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		global expand
		global sendResources
		expand = True
		sendResources = True

		banner()
		wait_resources = False
		print(_('In which city do you want to expand a building?'))
		city = chooseCity(session)
		cityId = city['id']
		building = getBuildingToExpand(session, cityId)
		if building is None:
			event.set()
			return

		current_level = building['level']
		if building['isBusy']:
			current_level += 1
		final_level = building['upgradeTo']

		# calculate the resources that are needed
		resourcesNeeded = getResourcesNeeded(session, city, building, current_level, final_level)
		if -1 in resourcesNeeded:
			event.set()
			return

		# calculate the resources that are missing
		missing = [0] * len(materials_names)
		for i in range(len(materials_names)):
			if city['recursos'][i] < resourcesNeeded[i]:
				missing[i] = resourcesNeeded[i] - city['recursos'][i]

		# show missing resources to the user
		if sum(missing) > 0:
			print(_('\nMissing:'))
			for i in range(len(materials_names)):
				if missing[i] == 0:
					continue
				name = materials_names[i].lower()
				print(_('{} of {}').format(addDot(missing[i]), name))
			print('')

			# if the user wants, send the resources from the selected cities
			print(_('Automatically transport resources? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				print(_('Proceed anyway? [Y/n]'))
				rta = read(values=['y', 'Y', 'n', 'N', ''])
				if rta.lower() == 'n':
					event.set()
					return
			else:
				wait_resources = True
				sendResourcesMenu(session, cityId, missing)
		else:
			print(_('\nYou have enough materials'))
			print(_('Proceed? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				event.set()
				return
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nUpgrade building\n')
	info = info + _('City: {}\nBuilding: {}. From {:d}, to {:d}').format(city['cityName'], building['name'], current_level, final_level)

	setInfoSignal(session, info)
	try:
		if expand:
			expandBuilding(session, cityId, building, wait_resources)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import re
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.naval import *
from ikabot.helpers.varios import *
from ikabot.helpers.resources import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import getCity

t = gettext.translation('getStatus',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def getStatus(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()
		color_arr = [bcolors.ENDC, bcolors.HEADER, bcolors.STONE, bcolors.BLUE, bcolors.WARNING]

		print(_('Ships {:d}/{:d}').format(getAvailableShips(session), getTotalShips(session)))

		print(_('\nOf which city do you want to see the state?'))
		city = chooseCity(session)
		banner()

		(wood, good, typeGood) = getProductionPerSecond(session, city['id'])
		print('\033[1m{}{}{}'.format(color_arr[int(typeGood)], city['cityName'], color_arr[0]))

		resources = city['recursos']
		storageCapacity = city['storageCapacity']
		color_resources = []
		for i in range(len(materials_names)):
			if resources[i] == storageCapacity:
				color_resources.append(bcolors.RED)
			else:
				color_resources.append(bcolors.ENDC)
		print(_('Storage:'))
		print(addDot(storageCapacity))
		print(_('Resources:'))
		for i in range(len(materials_names)):
			print('{} {}{}{} '.format(materials_names[i], color_resources[i], addDot(resources[i]), bcolors.ENDC), end='')
		print('')

		print(_('Production:'))
		print('{}:{} {}:{}'.format(materials_names[0], addDot(wood*3600), materials_names[typeGood], addDot(good*3600)))

		hasTavern = 'tavern' in [ building['building'] for building in city['position'] ]
		if hasTavern:
			consumption_per_hour = city['consumo']
			if consumption_per_hour == 0:
				print(_('{}{}Does not consume wine!{}').format(bcolors.RED, bcolors.BOLD, bcolors.ENDC))
			else:
				if typeGood == 1 and (good*3600) > consumption_per_hour:
					elapsed_time_run_out = '∞'
				else:
					consumption_per_second = Decimal(consumption_per_hour) / Decimal(3600)
					remaining_resources_to_consume = Decimal(resources[1]) / Decimal(consumption_per_second)
					elapsed_time_run_out = daysHoursMinutes(remaining_resources_to_consume)
				print(_('There is wine for: {}').format(elapsed_time_run_out))

		for building in [ building for building in city['position'] if building['name'] != 'empty' ]:
			if building['isMaxLevel'] is True:
				color = bcolors.BLACK
			elif building['canUpgrade'] is True:
				color = bcolors.GREEN
			else:
				color = bcolors.RED

			level = building['level']
			if level < 10:
				level = ' ' + str(level)
			else:
				level = str(level)
			if building['isBusy'] is True:
				level = level + '+'

			print(_('lv:{}\t{}{}{}').format(level, color, building['name'], bcolors.ENDC))

		enter()
		print('')
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
import gettext
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import *

t = gettext.translation('constructBuilding',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def constructBuilding(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()

		print(_('City where to build:'))
		city = chooseCity(session)
		banner()

		# list of free spaces in the selected city
		free_spaces = [ buildings for buildings in city['position'] if buildings['building'] == 'empty' ]

		# get a list of all the posible buildings that can be built
		buildings = []
		# different buildings can be built in different areas
		type_spaces = ['sea', 'land', 'shore', 'wall']
		for type_space in type_spaces:
			free_spaces_of_type = [ free_space for free_space in free_spaces if free_space['type'] == type_space ]
			if len(free_spaces_of_type) > 0:
				# we take any space in the desired area
				free_space_of_type = free_spaces_of_type[0]
				params = {'view': 'buildingGround', 'cityId': city['id'], 'position': free_space_of_type['position'], 'backgroundView': 'city', 'currentCityId': city['id'], 'actionRequest': actionRequest, 'ajax': '1'}
				buildings_response = session.post(params=params, noIndex=True)
				buildings_response = json.loads(buildings_response, strict=False)[1][1]
				if buildings_response == '':
					continue
				html = buildings_response[1]
				matches = re.findall(r'<li class="building (.+?)">\s*<div class="buildinginfo">\s*<div title="(.+?)"\s*class="buildingimg .+?"\s*onclick="ajaxHandlerCall\(\'.*?buildingId=(\d+)&', html)
				# add the buildings that can be built in this area
				for match in matches:
					buildings.append({'building': match[0], 'name': match[1], 'buildingId': match[2], 'type': type_space})

		if len(buildings) == 0:
			print(_('No building can be built.'))
			enter()
			event.set()
			return

		# show list of buildings to the user
		print(_('What building do you want to build?\n'))
		i = 0
		for building in buildings:
			i += 1
			print('({:d}) {}'.format(i, building['name']))
		selected_building_index = read(min=1, max=i)
		banner()

		# show posible positions for the selected building
		building = buildings[selected_building_index - 1]
		print('{}\n'.format(building['name']))
		options = [ position_id for position_id in city['position'] if position_id['building'] == 'empty' and position_id['type'] == building['type'] ]
		if len(options) == 1:
			option = options[0]
		else:
			print(_('In which position do you want to build?\n'))
			i = 0
			for option in options:
				i += 1
				print('({:d}) {}'.format(i, option['position']))
			selected_building_index = read(min=1, max=i)
			option = options[selected_building_index - 1]
			banner()

		# build it
		params = {'action': 'CityScreen', 'function': 'build', 'cityId': city['id'], 'position': option['position'], 'building': building['buildingId'], 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': 'buildingGround', 'actionRequest': actionRequest, 'ajax': '1'}
		buildings_response = session.post(params=params, noIndex=True)
		msg = json.loads(buildings_response, strict=False)[3][1][0]['text']
		print(msg)
		enter()
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
import ikabot.config as config
from ikabot.helpers.process import run

t = gettext.translation('checkForUpdate',
                        config.localedir,
                        languages=config.languages,
                        fallback=True)
_ = t.gettext

def checkForUpdate():
	upgrade = run('python3 -m pip search ikabot').decode('utf-8').strip()
	if 'ikabot' not in upgrade:
		return

	upgrade = upgrade.split('\n')
	if len(upgrade) != 3:
		return

	upgrade = upgrade[2]
	match = re.search(r' +.*?: +(.*)', upgrade)
	if match is None:
		return

	new = match.group(1)
	config.update_msg = _('[+] ikabot version {} is available\n').format(new)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import time
import json
import traceback
import threading
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import enter
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.varios import daysHoursMinutes
from ikabot.function.vacationMode import activateVacationMode

t = gettext.translation('alertAttacks',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def alertAttacks(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		if checkTelegramData(session) is False:
			event.set()
			return

		banner()
		default = 20
		minutes = read(msg=_('How often should I search for attacks?(min:3, default: {:d}): ').format(default), min=3, empty=True)
		if minutes == '':
			minutes = default
		print(_('I will check for attacks every {:d} minutes').format(minutes))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI check for attacks every {:d} minutes\n').format(minutes)
	setInfoSignal(session, info)
	try:
		do_it(session, minutes)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def respondToAttack(session):
	"""
	Parameters
	---------
	session : ikabot.web.session.Session
	"""

	# this allows the user to respond to an attack via telegram
	while True:
		time.sleep(60 * 3)
		responses = getUserResponse(session)
		for response in responses:
			# the response should be in the form of:
			# <pid>:<action number>
			rta = re.search(r'(\d+):?\s*(\d+)', response)
			if rta is None:
				continue

			pid = int(rta.group(1))
			action 	= int(rta.group(2))

			# if the pid doesn't match, we ignore it
			if pid != os.getpid():
				continue

			# currently just one action is supported
			if action == 1:
				# mv
				activateVacationMode(session)
			else:
				sendToBot(session, _('Invalid command: {:d}').format(action))

def do_it(session, minutes):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	minutes : int
	"""

	# this thread lets the user react to an attack once the alert is sent
	thread = threading.Thread(target=respondToAttack, args=(session,))
	thread.start()

	knownAttacks = []
	while True:
		# get the militaryMovements
		html = session.get()
		city_id = re.search(r'currentCityId:\s(\d+),', html).group(1)
		url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(city_id, actionRequest)
		movements_response = session.post(url)
		postdata = json.loads(movements_response, strict=False)
		militaryMovements = postdata[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
		timeNow = int(postdata[0][1]['time'])

		currentAttacks = []
		for militaryMovement in [ mov for mov in militaryMovements if mov['isHostile'] ]:
			event_id = militaryMovement['event']['id']
			currentAttacks.append(event_id)
			# if we already alerted this, do nothing
			if event_id not in knownAttacks:
				knownAttacks.append(event_id)

				# get information about the attack
				missionText = militaryMovement['event']['missionText']
				origin = militaryMovement['origin']
				target = militaryMovement['target']
				amountTroops = militaryMovement['army']['amount']
				amountFleets = militaryMovement['fleet']['amount']
				timeLeft = int(militaryMovement['eventTime']) - timeNow

				# send alert
				msg  = _('-- ALERT --\n')
				msg += missionText + '\n'
				msg += _('from the city {} of {}\n').format(origin['name'], origin['avatarName'])
				msg += _('a {}\n').format(target['name'])
				msg += _('{} units\n').format(amountTroops)
				msg += _('{} fleet\n').format(amountFleets)
				msg += _('arrival in: {}\n').format(daysHoursMinutes(timeLeft))
				msg += _('If you want to put the account in vacation mode send:\n')
				msg += _('{:d}:1').format(os.getpid())
				sendToBot(session, msg)

		# remove old attacks from knownAttacks
		for event_id in list(knownAttacks):
			if event_id not in currentAttacks:
				knownAttacks.remove(event_id)

		time.sleep(minutes * 60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.gui import enter
from ikabot.helpers.pedirInfo import getIdsOfCities
from ikabot.helpers.varios import wait

t = gettext.translation('loginDaily',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def loginDaily(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()
		print(_('I will enter every day.'))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI enter every day\n')
	setInfoSignal(session, info)
	try:
		do_it(session)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def do_it(session):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	"""
	while True:
		(ids, cities) = getIdsOfCities(session)
		cityId = ids[0]
		url = 'action=AvatarAction&function=giveDailyActivityBonus&dailyActivityBonusCitySelect={0}&startPageShown=1&detectedDevice=1&autoLogin=on&cityId={0}&activeTab=multiTab2&backgroundView=city&currentCityId={0}&actionRequest={1}&ajax=1'.format(cityId, actionRequest)
		session.post(url)
		wait(24*60*60, 1*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import enter, enter
from ikabot.helpers.varios import wait
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.pedirInfo import getIslandsIds
from ikabot.helpers.getJson import getIsland
from ikabot.helpers.process import set_child_mode

t = gettext.translation('searchForIslandSpaces',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def searchForIslandSpaces(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		if checkTelegramData(session) is False:
			event.set()
			return
		banner()
		print(_('I will search for new spaces each hour.'))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI search for new spaces each hour\n')
	setInfoSignal(session, info)
	try:
		do_it(session)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def do_it(session):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	"""

	# this dict will contain all the cities from each island
	# as they where in last scan
	cities_before_per_island = {}

	while True:
		# this is done inside the loop because the user may colonize in a new island
		islandsIds = getIslandsIds(session)
		for islandId in islandsIds:
			html = session.get(island_url + islandId)
			island = getIsland(html)
			# cities in the current island
			cities_now = [city_space for city_space in island['cities'] if city_space['type'] != 'empty'] #loads the islands non empty cities into ciudades

			# if we haven't scaned this island before,
			# save it and do nothing
			if islandId not in cities_before_per_island:
				cities_before_per_island[islandId] = cities_now.copy()
			else:
				cities_before = cities_before_per_island[islandId]

				# someone disappeared
				for city_before in cities_before:
					if city_before['id'] not in [ city_now['id'] for city_now in cities_now ]:
						# we didn't find the city_before in the cities_now
						msg = _('the city {} of the player {} disappeared in {} {}:{} {}').format(city_before['name'], city_before['Name'], materials_names[int(island['good'])], island['x'], island['y'], island['name'])
						sendToBot(session, msg)

				# someone colonised
				for city_now in cities_now:
					if city_now['id'] not in [ city_before['id'] for city_before in cities_before ]:
						# we didn't find the city_now in the cities_before
						msg = _('{} founded {} in {} {}:{} {}').format(city_now['Name'], city_now['name'], materials_names[int(island['good'])], island['x'], island['y'], island['name'])
						sendToBot(session, msg)

		wait(1*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import json
import random
import gettext
import sys
import ikabot.web.session
from ikabot.config import *
import ikabot.config as config
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read

t = gettext.translation('botComm',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def sendToBotDebug(session, msg, debugON):
	"""This function will send the ``msg`` argument passed to it as a message to the user on Telegram, only if ``debugOn`` is ``True``
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	msg : str
		a string representing the message to send to the user on Telegram
	debugON : bool
		a boolean indicating whether or not to send the message.
	"""
	if debugON:
		sendToBot(session, msg)

def sendToBot(session, msg, Token=False):
	"""This function will send the ``msg`` argument passed to it as a message to the user on Telegram
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	msg : str
		a string representing the message to send to the user on Telegram
	Token : bool
		a boolean indicating whether or not to attach the process id, the users server, world and Ikariam username to the message
	"""
	if Token is False:
		msg = 'pid:{}\n{}\n{}'.format(os.getpid(), config.infoUser, msg)
	sessionData = session.getSessionData()
	try:
		ikabot.web.session.normal_get('https://api.telegram.org/bot{}/sendMessage'.format(sessionData['telegram']['botToken']), params={'chat_id': sessionData['telegram']['chatId'], 'text': msg})
	except KeyError:
		pass

def telegramDataIsValid(session):
	"""This function checks whether or not there is any Telegram data stored in the .ikabot file
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object

	Returns
	-------
	valid : bool
		a boolean indicating whether or not there is any Telegram data stored in the .ikabot file

	"""
	sessionData = session.getSessionData()
	try:
		return len(sessionData['telegram']['botToken']) > 0 and len(sessionData['telegram']['chatId']) > 0
	except KeyError:
		return False

def getUserResponse(session):
	"""This function will retrieve a list of messages the user sent to the bot on Telegram.
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object

	Returns
	-------
	updates : list[str]
		a list containing all the messages the user sent to the bot on Telegram
	"""
	# returns messages that the user sends to the telegram bot
	sessionData = session.getSessionData()
	try:
		updates = ikabot.web.session.normal_get('https://api.telegram.org/bot{}/getUpdates'.format(sessionData['telegram']['botToken'])).text
		updates = json.loads(updates, strict=False)
		if updates['ok'] is False:
			return []
		updates = updates['result']
		# only return messages from the chatId of our user
		return [update['message']['text'] for update in updates if update['message']['chat']['id'] == int(sessionData['telegram']['chatId'])]
	except KeyError:
		return []

def checkTelegramData(session):
	"""This function doesn't actually check any data itself, that is done by the ``telegramDataIsValid`` function. This function returns ``True`` if there is any Telegram data in the .ikabot file, and if there is none, it will ask the user to input it.
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object

	Returns
	-------
	valid : bool
		a boolean indicating whether or not there is valid Telegram data in the .ikabot file.
	"""
	if telegramDataIsValid(session):
		return True
	else:
		banner()
		print(_('You must provide valid credentials to communicate by telegram.'))
		print(_('You require the token of the bot you are going to use and your chat_id'))
		print(_('For more information about how to obtain them read the readme at https://github.com/physics-sp/ikabot'))
		rta = read(msg=_('Will you provide the credentials now? [y/N]'), values=['y','Y','n', 'N', ''])
		if rta.lower() != 'y':
			return False
		else:
			return updateTelegramData(session)

def updateTelegramData(session, event=None, stdin_fd=None):
	"""This function asks the user to input the Telegram bot's token and the user's own Telegram chat id. After the user has inputted the neccessary data, this function will generate a random 4 digit number, send it to the user as a Telegram message using the token the user provided. It will then ask the user to input that number as validation.
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	event : multiprocessing.Event
		an event which, when fired, give back control of the terminal to the main process
	stdin_fd : int
		the standard input file descriptor passed to the function as a means of gaining control of the terminal

	Returns
	-------
	valid : bool
		a boolean indicating whether or not the Telegram data has been successfully updated
	"""
	if event is not None and stdin_fd is not None:
		sys.stdin = os.fdopen(stdin_fd) # give process access to terminal

	banner()
	botToken = read(msg=_('Bot\'s token:'))
	chat_id = read(msg=_('Chat_id:'))

	sessionData = session.getSessionData()
	sessionData['telegram'] = {}
	sessionData['telegram']['botToken'] = botToken.replace(' ', '').replace('.', '')
	sessionData['telegram']['chatId'] = chat_id
	session.setSessionData(sessionData)

	rand = random.randint(1000, 9999)
	msg = _('El token a ingresar es:{:d}').format(rand)
	sendToBot(session, msg, Token=True)

	rta = read(msg=_('A message was sent by telegram, did you receive it? [Y/n]'), values=['y','Y','n', 'N', ''])
	if rta.lower() == 'n':
		valid = False
	else:
		recibido = read(msg=_('Enter the received token in telegram:'), digit=True)
		if rand != recibido:
			print(_('The token is incorrect'))
			valid = False
		else:
			print(_('The token is correct'))
			valid = True

	if valid is False:
		sessionData['telegram']['botToken'] = ''
		sessionData['telegram']['chatId'] = ''
		session.setSessionData(sessionData)
		print(_('Check the credentials and re-supply them.'))
	else:
		print(_('The data was saved.'))
	enter()

	if event is not None and stdin_fd is not None:
		event.set() #give main process control before exiting
	return valid



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import gettext
import multiprocessing
from ikabot.config import *
from ikabot.web.session import *
from ikabot.helpers.gui import *
from ikabot.function.donate import donate
from ikabot.function.update import update
from ikabot.helpers.pedirInfo import read
from ikabot.function.getStatus import getStatus
from ikabot.function.donationBot import donationBot
from ikabot.helpers.botComm import updateTelegramData, telegramDataIsValid
from ikabot.helpers.process import updateProcessList
from ikabot.function.constructionList import constructionList
from ikabot.function.searchForIslandSpaces import searchForIslandSpaces
from ikabot.function.alertAttacks import alertAttacks
from ikabot.function.vacationMode import vacationMode
from ikabot.function.activateMiracle import activateMiracle
from ikabot.function.trainArmy import trainArmy
from ikabot.function.sellResources import sellResources
from ikabot.function.checkForUpdate import checkForUpdate
from ikabot.function.distributeResources import distributeResources
from ikabot.function.alertLowWine import alertLowWine
from ikabot.function.buyResources import buyResources
from ikabot.function.loginDaily import loginDaily
from ikabot.function.sendResources import sendResources
from ikabot.function.constructBuilding import constructBuilding
from ikabot.function.shipMovements import shipMovements
from ikabot.function.insertCookies import insertCookies

t = gettext.translation('command_line',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def menu(session, checkUpdate=True):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	checkUpdate : bool
	"""
	if checkUpdate:
		checkForUpdate()

	banner()

	process_list = updateProcessList(session)
	if len(process_list) > 0:
		print(_('Running tasks:'))
		for process in process_list:
			if len(process['proxies']) == 0:
				proxy = ''
			else:
				proxy = _('proxy: ') + str(process['proxies'])

			print(_('- pid: {} task: {} {}').format(process['pid'], process['action'], proxy))
		print('')

	menu_actions = [
					constructionList,
					sendResources,
					distributeResources,
					getStatus,
					donate,
					searchForIslandSpaces,
					loginDaily,
					alertAttacks,
					donationBot,
					alertLowWine,
					buyResources,
					sellResources,
					vacationMode,
					activateMiracle,
					trainArmy,
					shipMovements,
					constructBuilding,
					update,
					insertCookies,
					updateTelegramData
					]

	print(_('(0)  Exit'))
	print(_('(1)  Construction list'))
	print(_('(2)  Send resources'))
	print(_('(3)  Distribute resources'))
	print(_('(4)  Account status'))
	print(_('(5)  Donate'))
	print(_('(6)  Search for new spaces'))
	print(_('(7)  Login daily'))
	print(_('(8)  Alert attacks'))
	print(_('(9)  Donate automatically'))
	print(_('(10) Alert wine running out'))
	print(_('(11) Buy resources'))
	print(_('(12) Sell resources'))
	print(_('(13) Activate vacation mode'))
	print(_('(14) Activate miracle'))
	print(_('(15) Train army'))
	print(_('(16) See movements'))
	print(_('(17) Construct building'))
	print(_('(18) Update Ikabot'))
	print(_('(19) Insert cookies into Chrome'))
	if telegramDataIsValid(session):
		print(_('(20) Change the Telegram data'))
	else:
		print(_('(20) Enter the Telegram data'))

	total_options = len(menu_actions)
	selected = read(min=0, max=total_options)
	if selected != 0:
		try:
			selected -= 1
			event = multiprocessing.Event() #creates a new event
			process = multiprocessing.Process(target=menu_actions[selected], args=(session, event, sys.stdin.fileno()), name=menu_actions[selected].__name__)
			process.start()
			process_list.append({'pid': process.pid, 'proxies': session.s.proxies, 'action': menu_actions[selected].__name__ })
			updateProcessList(session, programprocesslist=process_list)
			event.wait() #waits for the process to fire the event that's been given to it. When it does  this process gets back control of the command line and asks user for more input
		except KeyboardInterrupt:
			pass
		menu(session, checkUpdate=False)
	else:
		if isWindows:
			# in unix, you can exit ikabot and close the terminal and the processes will continue to execute
			# in windows, you can exit ikabot but if you close the terminal, the processes will die
			print(_('Closing this console will kill the processes.'))
			enter()
		clear()
		os._exit(0) #kills the process which executes this statement, but it does not kill it's child processes

def init():
	home = 'HOMEPATH' if isWindows else 'HOME'
	os.chdir(os.getenv(home))
	if not os.path.isfile(ikaFile):
		open(ikaFile, 'w')
		os.chmod(ikaFile, 0o600)

def start():
	init()
	session = Session()
	try:
		menu(session)
	finally:
		clear()
		session.logout()

def main():
	try:
		start()
	except KeyboardInterrupt:
		clear()

if __name__ == '__main__':
	main()



import os
import random
import locale
import gettext

local = locale.setlocale(locale.LC_ALL, '')
if 'es_' in local:
	languages = ['es']
else:
	languages = ['en']
languages = ['none']

localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
t = gettext.translation('config',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

# only use common browsers
if random.randint(0, 1) == 0:
	user_agent = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36'
else:
	user_agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:77.0) Gecko/20100101 Firefox/77.0'

update_msg = ''

isWindows = os.name == 'nt'

proxy = False
if proxy:
	http_proxy  = "http://127.0.0.1:8080"
	https_proxy = "socks5://127.0.0.1:9050"
	proxyDict = {"http": http_proxy, "https": https_proxy}
else:
	proxyDict = {}

ids_cache = None
cities_cache = None
menu_cities = ''
infoUser = ''
ikaFile = '.ikabot'
city_url = 'view=city&cityId='
island_url = 'view=island&islandId='
prompt = ' >>  '
materials_names = [_('Wood'), _('Wine'), _('Marble'), _('Cristal'), _('Sulfur')]
materials_names_english = ['Wood', 'Wine', 'Marble', 'Cristal', 'Sulfur']
materials_names_tec = ['wood', 'wine', 'marble', 'glass', 'sulfur']
tradegoods_names = [_('Saw mill'), _('Vineyard'), _('Quarry'), _('Crystal Mine'), _('Sulfur Pit')]
ConnectionError_wait = 5 * 60
actionRequest = 'REQUESTID'

debugON_alertAttacks          = False
debugON_alertLowWine          = False
debugON_donationBot           = False
debugON_searchForIslandSpaces = False
debugON_loginDaily            = False
debugON_enviarVino            = False
debugON_sendResources         = False
debugON_constructionList      = False
debugON_buyResources          = False
debugON_activateMiracle       = False



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import sys
from ikabot.helpers.pedirInfo import read
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.naval import *
from ikabot.helpers.varios import *

t = gettext.translation('shipMovements',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def isHostile(movement):
	"""
	Parameters
	----------
	movement : dict

	Returns
	-------
	is hostile : bool
	"""
	if movement['army']['amount']:
		return True
	for mov in movement['fleet']['ships']:
		if mov['cssClass'] != 'ship_transport':
			return True
	return False

def shipMovements(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()

		print(_('Ships {:d}/{:d}\n').format(getAvailableShips(session), getTotalShips(session)))

		cityId = getCurrentCityId(session)
		url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(cityId, actionRequest)
		resp = session.post(url)
		resp = json.loads(resp, strict=False)
		movements = resp[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
		time_now = int(resp[0][1]['time'])

		if len(movements) == 0:
			print(_('There are no movements'))
			enter()
			event.set()
			return

		for movement in movements:

			color = ''
			if movement['isHostile']:
				color = bcolors.RED + bcolors.BOLD
			elif movement['isOwnArmyOrFleet']:
				color = bcolors.BLUE + bcolors.BOLD
			elif movement['isSameAlliance']:
				color = bcolors.GREEN + bcolors.BOLD

			origin      = '{} ({})'.format(movement['origin']['name'], movement['origin']['avatarName'])
			destination = '{} ({})'.format(movement['target']['name'], movement['target']['avatarName'])
			arrow       = '<-' if movement['event']['isFleetReturning'] else '->'
			time_left = int(movement['eventTime']) - time_now
			print('{}{} {} {}: {} ({}) {}'.format(color, origin, arrow, destination, movement['event']['missionText'], daysHoursMinutes(time_left), bcolors.ENDC))

			if movement['isHostile']:
				troops = movement['army']['amount']
				fleets = movement['fleet']['amount']
				print(_('Troops:{}\nFleets:{}').format(addDot(troops), addDot(fleets)))
			elif isHostile(movement):
				troops = movement['army']['amount']
				ships = 0
				fleets = 0
				for mov in movement['fleet']['ships']:
					if mov['cssClass'] == 'ship_transport':
						ships += int(mov['amount'])
					else:
						fleets += int(mov['amount'])
				print(_('Troops:{}\nFleets:{}\n Ships:{}').format(addDot(troops), addDot(fleets), addDot(ships)))
			else:
				assert len(materials_names) == 5
				total_load = 0
				for resource in movement['resources']:
					amount = resource['amount']
					tradegood = resource['cssClass'].split()[1]
					index = materials_names_tec.index(tradegood)
					tradegood = materials_names[index]
					total_load += int( amount.replace(',', '') )
					print(_('{} of {}').format(amount, tradegood))
				ships = int(math.ceil((Decimal(total_load) / Decimal(500))))
				print(_('{:d} Ships').format(ships))
		enter()
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
import json
from ikabot.config import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import *
from ikabot.helpers.resources import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *

t = gettext.translation('donate',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def donate(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()

		city = chooseCity(session)
		banner()

		woodAvailable = city['recursos'][0]

		islandId = city['islandId']
		html = session.get(island_url + islandId)
		island = getIsland(html)

		island_type = island['tipo']
		resource_name  = tradegoods_names[0]
		tradegood_name = tradegoods_names[int(island_type)]

		# get resource information
		url = 'view=resource&type=resource&islandId={0}&backgroundView=island&currentIslandId={0}&actionRequest={}&ajax=1'.format(islandId, actionRequest)
		resp = session.post(url)
		resp = json.loads(resp, strict=False)

		resourceLevel  = resp[0][1]['backgroundData']['resourceLevel']
		tradegoodLevel = resp[0][1]['backgroundData']['tradegoodLevel']
		resourceEndUpgradeTime = resp[0][1]['backgroundData']['resourceEndUpgradeTime']
		resourceUpgrading = resourceEndUpgradeTime > 0
		tradegoodEndUpgradeTime = resp[0][1]['backgroundData']['tradegoodEndUpgradeTime']
		tradegoodUpgrading = tradegoodEndUpgradeTime > 0

		if resourceUpgrading:
			resourceUpgradeMsg = _('(upgrading, ends in:{})').format(daysHoursMinutes(resourceEndUpgradeTime))
		else:
			resourceUpgradeMsg = ''
		if tradegoodUpgrading:
			tradegoodUpgradeMsg = _('(upgrading, ends in:{})').format(daysHoursMinutes(tradegoodEndUpgradeTime))
		else:
			tradegoodUpgradeMsg = ''

		html = resp[1][1][1]
		wood_total_needed, wood_donated = re.findall(r'<li class="wood">(.*?)</li>', html)
		wood_total_needed = wood_total_needed.replace(',', '').replace('.', '')
		wood_total_needed = int(wood_total_needed)
		wood_donated = wood_donated.replace(',', '').replace('.', '')
		wood_donated = int(wood_donated)

		if resourceUpgrading and tradegoodUpgrading:
			print(_('Both the {} (ends in:{}) and the {} (ends in:{}) are being upgraded rigth now.\n'.format(resource_name, daysHoursMinutes(resourceEndUpgradeTime), tradegood_name, daysHoursMinutes(tradegoodEndUpgradeTime))))
			enter()
			event.set()
			return

		print('{} lv:{} {}'.format(resource_name, resourceLevel, resourceUpgradeMsg))
		print('{} / {} ({}%)\n'.format(addDot(wood_donated), addDot(wood_total_needed), addDot(int((100 * wood_donated) / wood_total_needed))))

		# get tradegood information
		url = 'view=tradegood&type={0}&islandId={1}&backgroundView=island&currentIslandId={1}&actionRequest={}&ajax=1'.format(island_type, islandId, actionRequest)
		resp = session.post(url)

		resp = json.loads(resp, strict=False)
		html = resp[1][1][1]
		tradegood_total_needed, tradegood_donated = re.findall(r'<li class="wood">(.*?)</li>', html)
		tradegood_total_needed = tradegood_total_needed.replace(',', '').replace('.', '')
		tradegood_total_needed = int(tradegood_total_needed)
		tradegood_donated = tradegood_donated.replace(',', '').replace('.', '')
		tradegood_donated = int(tradegood_donated)

		print('{} lv:{} {}'.format(tradegood_name, tradegoodLevel, tradegoodUpgradeMsg))
		print('{} / {} ({}%)\n'.format(addDot(tradegood_donated), addDot(tradegood_total_needed), addDot(int((100 * tradegood_donated) / tradegood_total_needed))))

		print(_('Wood available:{}\n').format(addDot(woodAvailable)))

		if resourceUpgrading is False and tradegoodUpgrading is False:
			msg = _('Donate to {} (1) or {} (2)?:').format(resource_name, tradegood_name)
			donation_type = read(msg=msg, min=1, max=2)
			name = resource_name if donation_type == 1 else tradegood_name
			print('')
		else:
			if resourceUpgrading is False and tradegoodUpgrading is True:
				donation_type = 1
				name = resource_name
			else:
				donation_type = 2
				name = tradegood_name
			print('Donate to:{}\n'.format(name))

		donation_type = 'resource' if donation_type == 1 else 'tradegood'

		amount = read(min=0, max=woodAvailable, msg=_('Amount:'))
		if amount == 0:
			event.set()
			return
		print(_('Will donate {} to the {}?').format(addDot(amount), name))
		print(_('\nProceed? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			return

		# do the donation
		session.post(payloadPost={'islandId': islandId, 'type': donation_type, 'action': 'IslandScreen', 'function': 'donate', 'donation': amount, 'backgroundView': 'island', 'templateView': 'resource', 'actionRequest': actionRequest, 'ajax': '1'})

		print('\nDonation successful.')
		enter()
		event.set()
		return
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import wait
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCity
from ikabot.helpers.resources import getAvailableResources

t = gettext.translation('donationBot',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def donationBot(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	try:
		banner()
		(cities_ids, cities) = getIdsOfCities(session)
		cities_dict = {}
		initials = [ material_name[0] for material_name in materials_names ]
		for cityId in cities_ids:
			tradegood = cities[cityId]['tradegood']
			initial = initials[int(tradegood)]
			print(_('In {} ({}), Do you wish to donate to the forest, to the trading good or neither? [f/t/n]').format(cities[cityId]['name'], initial))
			f = _('f')
			t = _('t')
			n = _('n')

			rta = read(values=[f, f.upper(), t, t.upper(), n, n.upper()])
			if rta.lower() == f:
				donation_type = 'resource'
			elif rta.lower() == t:
				donation_type = 'tradegood'
			else:
				donation_type = None
				percentage = None

			if donation_type is not None:
				print(_('What is the maximum percentage of your storage capacity that you whish to keep occupied? (the resources that exceed it, will be donated) (default: 80%)'))
				percentage = read(min=0, max=100, empty=True)
				if percentage == '':
					percentage = 80
				elif percentage == 100: # if the user is ok with the storage beeing totally full, don't donate at all
					donation_type = None

			cities_dict[cityId] = {'donation_type': donation_type, 'percentage': percentage}

		print(_('I will donate every day.'))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI donate every day\n')
	setInfoSignal(session, info)
	try:
		do_it(session, cities_ids, cities_dict)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def do_it(session, cities_ids, cities_dict):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	cities_ids : list[int]
	cities_dict : dict[int, dict]
	"""
	for cityId in cities_ids:
		html = session.get(city_url + cityId)
		city = getCity(html)
		cities_dict[cityId]['island'] = city['islandId']

	while True:
		for cityId in cities_ids:
			donation_type = cities_dict[cityId]['donation_type']
			if donation_type is None:
				continue

			# get the storageCapacity and the wood this city has
			html = session.get(city_url + cityId)
			city = getCity(html)
			wood = city['recursos'][0]
			storageCapacity = city['storageCapacity']

			# get the percentage
			percentage = cities_dict[cityId]['percentage']
			percentage /= 100

			# calculate what is the amount of wood that should be preserved
			max_wood = storageCapacity * percentage
			max_wood = int(max_wood)

			# calculate the wood that is exceeding the percentage
			to_donate = wood - max_wood
			if to_donate <= 0:
				continue

			islandId = cities_dict[cityId]['island']

			# donate
			session.post(payloadPost={'islandId': islandId, 'type': donation_type, 'action': 'IslandScreen', 'function': 'donate', 'donation': to_donate, 'backgroundView': 'island', 'templateView': donation_type, 'actionRequest': actionRequest, 'ajax': '1'})

		msg = _('I donated automatically.')
		sendToBotDebug(session, msg, debugON_donationBot)

		# sleep a day
		wait(24*60*60, maxrandom=60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import sys
import json
import time
import random
import getpass
import datetime
import gettext
import requests
import base64
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import banner
from ikabot.helpers.aesCipher import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCity

t = gettext.translation('session',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext


class Session:
	def __init__(self):
		self.logfile = '/tmp/debug.txt'
		self.log = False
		self.padre = True
		self.logged = False
		self.__login()

	def __log(self, msg):
		if self.log is False:
			return
		now = datetime.datetime.now()
		entry = '{}:{}:{}\t{:d}: {}\n'.format(now.hour, now.minute, now.second, os.getpid(), msg)
		fh = open(self.logfile, 'a')
		fh.write(entry)
		fh.close()

	def __genRand(self):
		return hex(random.randint(0, 65535))[2:]

	def __genCookie(self):
		return self.__genRand() + self.__genRand() + hex(int(round(time.time() * 1000)))[2:] + self.__genRand() + self.__genRand()

	def __fp_eval_id(self):
		return self.__genRand() + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + self.__genRand() + self.__genRand()

	def __logout(self, html):
		if html is not None:
			idCiudad = getCity(html)['id']
			token = re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)
			urlLogout = 'action=logoutAvatar&function=logout&sideBarExt=undefined&startPageShown=1&detectedDevice=1&cityId={0}&backgroundView=city&currentCityId={0}&actionRequest={1}'.format(idCiudad, token)
			self.s.get(self.urlBase + urlLogout)

	def __isInVacation(self, html):
		return 'nologin_umod' in html

	def __isExpired(self, html):
		return 'index.php?logout' in html

	def __updateCookieFile(self, primero=False, nuevo=False, salida=False):
		sessionData = self.getSessionData()

		if primero is True:
			cookie_dict = dict(self.s.cookies.items())
			sessionData['cookies'] = cookie_dict
			sessionData['num_sessions'] = 1

		elif nuevo is True:
			try:
				sessionData['num_sessions'] += 1
			except KeyError:
				sessionData['num_sessions'] = 1

		elif salida is True:
			try:
				if sessionData['num_sessions'] == 1:
					html = self.s.get(self.urlBase).text
					if self.__isExpired(html) is False:
						self.__logout(html)
			except KeyError:
				return
			sessionData['num_sessions'] -= 1

		self.setSessionData(sessionData)

	def __getCookie(self, sessionData=None):
		if sessionData is None:
			sessionData = self.getSessionData()
		try:
			assert sessionData['num_sessions'] > 0
			cookie_dict = sessionData['cookies']
			self.s = requests.Session()
			#self.s.proxies = proxyDict
			self.s.headers.clear()
			self.s.headers.update(self.headers)
			requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
			self.__updateCookieFile(nuevo=True)
		except (KeyError, AssertionError):
			self.__login(3)

	def __login(self, retries=0):
		self.__log('__login()')
		if not self.logged:
			banner()

			self.mail = read(msg=_('Mail:'))
			self.password = getpass.getpass(_('Password:'))

			banner()

		self.s = requests.Session()
		#self.s.proxies = proxyDict

		# get gameEnvironmentId and platformGameId
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/config/configuration.js')

		js = r.text
		gameEnvironmentId = re.search(r'"gameEnvironmentId":"(.*?)"', js)
		if gameEnvironmentId is None:
			exit('gameEnvironmentId not found')
		gameEnvironmentId = gameEnvironmentId.group(1)
		platformGameId = re.search(r'"platformGameId":"(.*?)"', js)
		if platformGameId is None:
			exit('platformGameId not found')
		platformGameId = platformGameId.group(1)

		# get __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/js/connect.js')
		html = r.text
		captcha = re.search(r'Attention Required', html)
		if captcha is not None:
			exit('Captcha error!')

		# update __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/config')

		__fp_eval_id_1 = self.__fp_eval_id()
		__fp_eval_id_2 = self.__fp_eval_id()

		# get pc_idt cookie
		self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'VISIT', 'replacement_kid': '', 'fp_eval_id': __fp_eval_id_1, 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fingerprint': '2175408712', 'fp_exec_time': '1.00'}
		r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

		# update pc_idt cookie
		self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'fp_eval', 'fp_eval_id': __fp_eval_id_2, 'fingerprint': '2175408712', 'fp2_config_id': '1', 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fp2_value': '921af958be7cf2f76db1e448c8a5d89d', 'fp2_exec_time': '96.00'}
		r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

		# options req (not really needed)
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate','Access-Control-Request-Method': 'POST','Access-Control-Request-Headers': 'content-type,tnt-installation-id','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.options('https://gameforge.com/api/v1/auth/thin/sessions')

		# send creds
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate, br','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','TNT-Installation-Id': '','Content-Type': 'application/json','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'keep-alive','Pragma': 'no-cache','Cache-Control': 'no-cache','TE': 'Trailers'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {"identity": self.mail, "password": self.password, "locale":"es_AR", "gfLang":"ar", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": "false"}
		r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)
		if r.status_code == 403:
			exit(_('Wrong email or password\n'))

		# get the authentication token and set the cookie
		ses_json = json.loads(r.text, strict=False)
		auth_token = ses_json['token']
		cookie_obj = requests.cookies.create_cookie(domain='.gameforge.com', name='gf-token-production', value=auth_token)
		self.s.cookies.set_cookie(cookie_obj)

		# get accounts
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/accounts')
		accounts = json.loads(r.text, strict=False)

		# get servers
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/servers')
		servers = json.loads(r.text, strict=False)

		if not self.logged:

			if len([ account for account in accounts if account['blocked'] is False ]) == 1:
				self.account  = [ account for account in accounts if account['blocked'] is False ][0]
			else:
				print(_('With which account do you want to log in?\n'))

				max_name = max( [ len(account['name']) for account in accounts if account['blocked'] is False ] )
				i = 0
				for account in [ account for account in accounts if account['blocked'] is False ]:
					server = account['server']['language']
					mundo = account['server']['number']
					world = [ srv['name'] for srv in servers if srv['language'] == server and srv['number'] == mundo ][0]
					i += 1
					pad = ' ' * (max_name - len(account['name']))
					print('({:d}) {}{} [{} - {}]'.format(i, account['name'], pad, server, world))
				num = read(min=1, max=i)
				self.account  = [ account for account in accounts if account['blocked'] is False ][num - 1]
			self.username = self.account['name']
			self.servidor = self.account['server']['language']
			self.mundo    = str(self.account['server']['number'])
			self.word     = [ srv['name'] for srv in servers if srv['language'] == self.servidor and srv['number'] == int(self.mundo) ][0]
			config.infoUser = _('Server:{}').format(self.servidor)
			config.infoUser += _(', World:{}').format(self.word)
			config.infoUser += _(', Player:{}').format(self.username)
			banner()

		self.host = 's{}-{}.ikariam.gameforge.com'.format(self.mundo, self.servidor)
		self.urlBase = 'https://{}/index.php?'.format(self.host)

		self.headers = {'Host': self.host, 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://{}'.format(self.host), 'X-Requested-With': 'XMLHttpRequest', 'Origin': 'https://{}'.format(self.host), 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}

		self.cipher = AESCipher(self.mail, self.username, self.password)
		sessionData = self.getSessionData()

		used_old_cookies = False
		# if there are cookies stored, try to use them
		if 'num_sesiones' in sessionData and sessionData['num_sesiones'] > 0 and self.logged is False:
			# create a new temporary session object
			old_s = requests.Session()
			# set the headers
			old_s.headers.clear()
			old_s.headers.update(self.headers)
			# set the cookies to test
			cookie_dict = sessionData['cookies']
			requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=old_s.cookies, overwrite=True)
			# make a request to check the connection
			old_s.proxies = proxyDict
			html = old_s.get(self.urlBase).text
			old_s.proxies = {}

			cookies_are_valid = self.__isExpired(html) is False
			if cookies_are_valid:
				self.__log('using old cookies')
				used_old_cookies = True
				# assign the old cookies to the session object
				requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
				# set the headers
				self.s.headers.clear()
				self.s.headers.update(self.headers)

		# login as normal and get new cookies
		if used_old_cookies is False:
			self.__log('using new cookies')
			resp = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/loginLink?id={}&server[language]={}&server[number]={}'.format(self.account['id'], self.servidor, self.mundo)).text
			resp = json.loads(resp, strict=False)
			if 'url' not in resp:
				if retries > 0:
					return self.__login(retries-1)
				else:
					msg = 'Login Error: ' + str(resp)
					if self.padre:
						print(msg)
						exit()
					else:
						exit(msg)

			url = resp['url']
			match = re.search(r'https://s\d+-\w{2}\.ikariam\.gameforge\.com/index\.php\?', url)
			if match is None:
				exit('Error')

			# set the headers
			self.s.headers.clear()
			self.s.headers.update(self.headers)

			# use the new cookies instead, invalidate the old ones
			self.s.proxies = proxyDict
			html = self.s.get(url).text
			self.s.proxies = {}

		if self.__isInVacation(html):
			msg = _('The account went into vacation mode')
			if self.padre:
				print(msg)
			else:
				sendToBot(self, msg)
			os._exit(0)
		if self.__isExpired(html):
			if retries > 0:
				return self.__login(retries-1)
			if self.padre:
				msg = _('Login error.')
				print(msg)
				os._exit(0)
			raise Exception('Couldn\'t log in')

		if used_old_cookies:
			self.__updateCookieFile(nuevo=True)
		else:
			self.__updateCookieFile(primero=True)

		self.logged = True

	def __backoff(self):
		self.__log('__backoff()')
		if self.padre is False:
			time.sleep(5 * random.randint(0, 10))

	def __sessionExpired(self):
		self.__log('__sessionExpired()')
		self.__backoff()

		sessionData = self.getSessionData()

		try:
			if sessionData['num_sessions'] > 0 and self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
				self.__getCookie(sessionData)
			else:
				try:
					self.__login(3)
				except Exception:
					self.__sessionExpired()
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def __checkCookie(self):
		self.__log('__checkCookie()')
		sessionData = self.getSessionData()

		try:
			if sessionData['num_sessions'] > 0:
				if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
					self.__getCookie(sessionData)
			else:
				try:
					self.__login(3)
				except Exception:
					self.__sessionExpired()
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def __token(self):
		"""Generates a valid actionRequest token from the session
		Returns
		-------
		token : str
			a string representing a valid actionRequest token
		"""
		html = self.get()
		return re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)

	def get(self, url='', params={}, ignoreExpire=False, noIndex=False):
		"""Sends get request to ikariam
		Parameters
		----------
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the get request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the get request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url params and sending the get request

		Returns
		-------
		html : str
			response from the server
		"""
		self.__checkCookie()

		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		self.__log('get({}), params:{}'.format(url, str(params)))
		while True:
			try:
				self.s.proxies = proxyDict
				html = self.s.get(url, params=params).text #this isn't recursion, this get is different from the one it's in
				self.s.proxies = {}
				if ignoreExpire is False:
					assert self.__isExpired(html) is False
				return html
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def post(self, url='', payloadPost={}, params={}, ignoreExpire=False, noIndex=False):
		"""Sends post request to ikariam
		Parameters
		----------
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		payloadPost : dict
			dictionary containing key-value pairs which represent the payload of the post request
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the post request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the post request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url and params and sending the post request

		Returns
		-------
		html : str
			response from the server
		"""
		url_original = url
		payloadPost_original = payloadPost
		params_original = params
		self.__checkCookie()

		# add the request id
		token = self.__token()
		url = url.replace(actionRequest, token)
		if 'actionRequest' in payloadPost:
			payloadPost['actionRequest'] = token
		if 'actionRequest' in params:
			params['actionRequest'] = token

		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		self.__log('post({}), data={}'.format(url, str(payloadPost)))
		while True:
			try:
				self.s.proxies = proxyDict
				resp = self.s.post(url, data=payloadPost, params=params).text
				self.s.proxies = {}
				if ignoreExpire is False:
					assert self.__isExpired(resp) is False
				if 'TXT_ERROR_WRONG_REQUEST_ID' in resp:
					self.__log(_('got TXT_ERROR_WRONG_REQUEST_ID'))
					return self.post(url=url_original, payloadPost=payloadPost_original, params=params_original, ignoreExpire=ignoreExpire, noIndex=noIndex)
				self.__log(resp)
				return resp
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def login(self):
		"""This function doesn't actually log into ikariam, it only increments a number in the .ikabot file which represents the number currently running sessions
		"""
		self.__log('login({})')
		self.__updateCookieFile(nuevo=True)

	def logout(self):
		"""This function decrements a number in the .ikabot file representing the number of currently running sessions. If this number is 1, it will attempt to completely log out of ikariam
		"""
		self.__log('logout({})')
		self.__updateCookieFile(salida=True)
		if self.padre is False:
			os._exit(0)

	def setSessionData(self, sessionData):
		"""Encrypts relevant session data and writes it to the .ikabot file
		Parameters
		----------
		sessionData : dict
			dictionary containing relevant session data, data is written to file using AESCipher.setSessionData
		"""
		self.cipher.setSessionData(self, sessionData)

	def getSessionData(self):
		"""Gets relevant session data from the .ikabot file
		"""
		return self.cipher.getSessionData(self)

def normal_get(url, params={}):
	"""Sends a get request to provided url
	Parameters
	----------
	url : str
		a string representing the url to which to send the get request
	params : dict
		a dictionary containing key-value pairs which represent the parameters of the get request

	Returns
	-------
	response : requests.Response
		a requests.Response object which represents the webservers response. For more information on requests.Response refer to https://requests.readthedocs.io/en/master/api/#requests.Response
	"""
	try:

		return requests.get(url, params=params)

	except requests.exceptions.ConnectionError:
		sys.exit(_('Internet connection failed'))



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import gettext
import json
import sys
from ikabot.helpers.gui import *
from ikabot.config import *

t = gettext.translation('insertCookies',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def insertCookies(session, event, stdin_fd):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	"""
	sys.stdin = os.fdopen(stdin_fd)
	banner()
	try:
		cookies = json.dumps(session.getSessionData()['cookies'])
		cookies_js = 'cookies={};i=0;for(let cookie in cookies){{document.cookie=Object.keys(cookies)[i]+\"=\"+cookies[cookie];i++}}'.format(cookies)
		print("""To prevent ikabot from logging you out while playing Ikariam do the following:
		1. While you're in Ikariam on the "Your session has expired screen" open chrome developer tools by pressing CTRL + SHIFT + I
		2. Open the Application tab in developer tools (You may need to press CONTROL + SHIFT + D to be able to see it)
		3. Click Cookies
		4. Click on the tab named s[number]-[region].ikariam.gameforge.com 
		5. Delete all the cookies by clicking on them and pressing DEL on your keyboard
		6. Open the Console tab in developer tools
		7. Copy the text below, paste it into the console and press enter
		8. Press F5
		""")
		print(cookies_js)
		enter()
		event.set()
	except KeyboardInterrupt:
		event.set()
		return
